@@ -23,6 +23,8 @@
 #include <range/v3/iterator/traits.hpp>
 #include <range/v3/utility/static_const.hpp>
 
+#include <range/v3/detail/disable_warnings.hpp>
+
 namespace ranges
 {
     /// \cond
@@ -59,7 +61,7 @@ namespace ranges
             ///
             /// range-based version of the `upper_bound` std algorithm
             ///
-            /// \pre `Rng` is a model of the `Range` concept
+            /// \pre `Rng` is a model of the `range` concept
             template<typename I, typename V, typename C = less, typename P = identity>
             auto operator()(I first, iter_difference_t<I> d, V const & val, C pred = C{},
                             P proj = P{}) const -> CPP_ret(I)( //
@@ -77,4 +79,6 @@ namespace ranges
     } // namespace aux
 } // namespace ranges
 
-#endif // include guard
+#include <range/v3/detail/reenable_warnings.hpp>
+
+#endif
