@@ -27,16 +27,18 @@
 #include <range/v3/range/traits.hpp>
 #include <range/v3/utility/static_const.hpp>
 
+#include <range/v3/detail/disable_warnings.hpp>
+
 namespace ranges
 {
     /// \addtogroup group-algorithms
     /// @{
-    RANGES_BEGIN_NIEBLOID(find_if)
+    RANGES_FUNC_BEGIN(find_if)
         /// \brief template function \c find
         ///
         /// range-based version of the \c find std algorithm
         ///
-        /// \pre `Rng` is a model of the `Range` concept
+        /// \pre `Rng` is a model of the `range` concept
         /// \pre `I` is a model of the `input_iterator` concept
         /// \pre `S` is a model of the `sentinel_for<I>` concept
         /// \pre `P` is a model of the `invocable<V>` concept, where `V` is the
@@ -44,8 +46,8 @@ namespace ranges
         /// \pre `F` models `predicate<X>`, where `X` is the result type
         ///      of `invocable<P, V>`
         template<typename I, typename S, typename F, typename P = identity>
-        auto RANGES_FUN_NIEBLOID(find_if)(I first, S last, F pred, P proj = P{}) //
-            ->CPP_ret(I)(                                                        //
+        auto RANGES_FUNC(find_if)(I first, S last, F pred, P proj = P{}) //
+            ->CPP_ret(I)(                                                //
                 requires input_iterator<I> && sentinel_for<S, I> &&
                 indirect_unary_predicate<F, projected<I, P>>)
         {
@@ -57,7 +59,7 @@ namespace ranges
 
         /// \overload
         template<typename Rng, typename F, typename P = identity>
-        auto RANGES_FUN_NIEBLOID(find_if)(Rng && rng, F pred, P proj = P{})
+        auto RANGES_FUNC(find_if)(Rng && rng, F pred, P proj = P{})
             ->CPP_ret(safe_iterator_t<Rng>)( //
                 requires input_range<Rng> &&
                 indirect_unary_predicate<F, projected<iterator_t<Rng>, P>>)
@@ -65,7 +67,7 @@ namespace ranges
             return (*this)(begin(rng), end(rng), std::move(pred), std::move(proj));
         }
 
-    RANGES_END_NIEBLOID(find_if)
+    RANGES_FUNC_END(find_if)
 
     namespace cpp20
     {
@@ -74,4 +76,6 @@ namespace ranges
     /// @}
 } // namespace ranges
 
-#endif // include guard
+#include <range/v3/detail/reenable_warnings.hpp>
+
+#endif
