@@ -22,6 +22,8 @@
 #include <range/v3/iterator/traits.hpp>
 #include <range/v3/range/concepts.hpp>
 
+#include <range/v3/detail/disable_warnings.hpp>
+
 namespace ranges
 {
     /// \addtogroup group-iterator
@@ -138,14 +140,13 @@ namespace ranges
 #else
     private:
         template<typename I>
-        static constexpr void n_(I & i, iter_difference_t<I> n,
-                                 detail::input_iterator_tag_);
+        static constexpr void n_(I & i, iter_difference_t<I> n, std::input_iterator_tag);
         template<typename I>
         static constexpr void n_(I & i, iter_difference_t<I> n,
-                                 detail::bidirectional_iterator_tag_);
+                                 std::bidirectional_iterator_tag);
         template<typename I>
         static constexpr void n_(I & i, iter_difference_t<I> n,
-                                 detail::random_access_iterator_tag_);
+                                 std::random_access_iterator_tag);
         template<typename I, typename S>
         static constexpr void to_impl_(I & i, S s, sentinel_tag);
         template<typename I, typename S>
@@ -157,11 +158,11 @@ namespace ranges
         template<typename I, typename S>
         static constexpr iter_difference_t<I> bounded_(I & it, iter_difference_t<I> n,
                                                        S bound, sentinel_tag,
-                                                       detail::input_iterator_tag_);
+                                                       std::input_iterator_tag);
         template<typename I>
-        static constexpr iter_difference_t<I> bounded_(
-            I & it, iter_difference_t<I> n, I bound, sentinel_tag,
-            detail::bidirectional_iterator_tag_);
+        static constexpr iter_difference_t<I> bounded_(I & it, iter_difference_t<I> n,
+                                                       I bound, sentinel_tag,
+                                                       std::bidirectional_iterator_tag);
         template<typename I, typename S, typename Concept>
         static constexpr iter_difference_t<I> bounded_(I & it, iter_difference_t<I> n,
                                                        S bound, sized_sentinel_tag,
@@ -209,8 +210,7 @@ namespace ranges
 
 #if RANGES_CXX_IF_CONSTEXPR < RANGES_CXX_IF_CONSTEXPR_17
     template<typename I>
-    constexpr void advance_fn::n_(I & i, iter_difference_t<I> n,
-                                  detail::input_iterator_tag_)
+    constexpr void advance_fn::n_(I & i, iter_difference_t<I> n, std::input_iterator_tag)
     {
         RANGES_EXPECT(n >= 0);
         for(; n > 0; --n)
@@ -218,7 +218,7 @@ namespace ranges
     }
     template<typename I>
     constexpr void advance_fn::n_(I & i, iter_difference_t<I> n,
-                                  detail::bidirectional_iterator_tag_)
+                                  std::bidirectional_iterator_tag)
     {
         if(n > 0)
             for(; n > 0; --n)
@@ -229,7 +229,7 @@ namespace ranges
     }
     template<typename I>
     constexpr void advance_fn::n_(I & i, iter_difference_t<I> n,
-                                  detail::random_access_iterator_tag_)
+                                  std::random_access_iterator_tag)
     {
         i += n;
     }
@@ -260,7 +260,7 @@ namespace ranges
     template<typename I, typename S>
     constexpr iter_difference_t<I> advance_fn::bounded_(I & it, iter_difference_t<I> n,
                                                         S bound, sentinel_tag,
-                                                        detail::input_iterator_tag_)
+                                                        std::input_iterator_tag)
     {
         RANGES_EXPECT(0 <= n);
         for(; 0 != n && it != bound; --n)
@@ -268,9 +268,9 @@ namespace ranges
         return n;
     }
     template<typename I>
-    constexpr iter_difference_t<I> advance_fn::bounded_(
-        I & it, iter_difference_t<I> n, I bound, sentinel_tag,
-        detail::bidirectional_iterator_tag_)
+    constexpr iter_difference_t<I> advance_fn::bounded_(I & it, iter_difference_t<I> n,
+                                                        I bound, sentinel_tag,
+                                                        std::bidirectional_iterator_tag)
     {
         if(0 <= n)
             for(; 0 != n && it != bound; --n)
@@ -414,7 +414,8 @@ namespace ranges
         template<typename I, typename S>
         static constexpr iter_difference_t<I> impl_i(I first, S last, sentinel_tag)
         {
-            return iter_enumerate(static_cast<I &&>(first), static_cast<S &&>(last)).first;
+            return iter_enumerate(static_cast<I &&>(first), static_cast<S &&>(last))
+                .first;
         }
         template<typename I, typename S>
         static constexpr iter_difference_t<I> impl_i(I first, S last, sized_sentinel_tag)
@@ -583,7 +584,7 @@ namespace ranges
             return enumerate(rng).first;
         }
         template<typename Rng>
-        constexpr static range_difference_t<Rng> impl_r(Rng & rng, sized_range_tag)
+        static constexpr range_difference_t<Rng> impl_r(Rng & rng, sized_range_tag)
         {
             return static_cast<range_difference_t<Rng>>(size(rng));
         }
@@ -653,4 +654,6 @@ namespace ranges
     /// @}
 } // namespace ranges
 
+#include <range/v3/detail/reenable_warnings.hpp>
+
 #endif // RANGES_V3_ITERATOR_OPERATIONS_HPP
