@@ -13,6 +13,8 @@
 
 #include <range/v3/detail/config.hpp>
 
+#include <range/v3/detail/disable_warnings.hpp>
+
 namespace ranges
 {
     template<typename Target, typename Source>
@@ -39,4 +41,6 @@ namespace ranges
     }
 } // namespace ranges
 
+#include <range/v3/detail/reenable_warnings.hpp>
+
 #endif // RANGES_V3_UTILITY_POLYMORPHIC_CAST_HPP
