@@ -66,6 +66,8 @@
 #include <mutex>
 #endif
 
+#include <range/v3/detail/disable_warnings.hpp>
+
 RANGES_DIAGNOSTIC_PUSH
 RANGES_DIAGNOSTIC_IGNORE_CXX17_COMPAT
 
@@ -74,20 +76,21 @@ namespace ranges
     /// \addtogroup group-numerics
     /// @{
     // clang-format off
-    CPP_def
-    (
-        template(typename Gen)
-        concept uniform_random_bit_generator,
-            requires(int)
-            (
-                Gen::min(),
-                Gen::max(),
-                concepts::requires_<same_as<invoke_result_t<Gen&>, decltype(Gen::min())>>,
-                concepts::requires_<same_as<invoke_result_t<Gen&>, decltype(Gen::max())>>
-            ) &&
-            invocable<Gen &> &&
-            unsigned_integral<invoke_result_t<Gen&>>
+    template<typename Gen>
+    CPP_concept_fragment(uniform_random_bit_generator_, (Gen),
+        unsigned_integral<invoke_result_t<Gen &>> &&
+        same_as<invoke_result_t<Gen &>, decltype(Gen::min())> &&
+        same_as<invoke_result_t<Gen &>, decltype(Gen::max())>
     );
+    template<typename Gen>
+    CPP_concept_bool uniform_random_bit_generator =
+        invocable<Gen &> &&
+        CPP_requires ((int)) //
+        (
+            CPP_type(Gen)::min(),
+            CPP_type(Gen)::max()
+        ) &&
+        CPP_fragment(ranges::uniform_random_bit_generator_, Gen);
     // clang-format on
     /// @}
 
@@ -482,4 +485,6 @@ namespace ranges
 
 RANGES_DIAGNOSTIC_POP
 
+#include <range/v3/detail/reenable_warnings.hpp>
+
 #endif
