@@ -23,6 +23,8 @@
 #include <range/v3/view/transform.hpp>
 #include <range/v3/view/view.hpp>
 
+#include <range/v3/detail/disable_warnings.hpp>
+
 namespace ranges
 {
     /// \addtogroup group-views
@@ -51,9 +53,11 @@ namespace ranges
 
         /// \relates addressof_fn
         /// \ingroup group-views
-        RANGES_INLINE_VARIABLE(view<addressof_fn>, addressof)
+        RANGES_INLINE_VARIABLE(view_closure<addressof_fn>, addressof)
     } // namespace views
     /// @}
 } // namespace ranges
 
+#include <range/v3/detail/reenable_warnings.hpp>
+
 #endif // RANGES_V3_VIEW_ADDRESSOF_HPP
