@@ -34,6 +34,8 @@
 #include <range/v3/view/facade.hpp>
 #include <range/v3/view/view.hpp> // for dereference_fn
 
+#include <range/v3/detail/disable_warnings.hpp>
+
 namespace ranges
 {
     /// \cond
@@ -75,46 +77,52 @@ namespace ranges
     } // namespace detail
       /// \endcond
 
+    /// \addtogroup group-views
+    /// @{
+
     // clang-format off
-    CPP_def
-    (
-        template(typename...Views)
-        (concept cartesian_produce_view_can_const)(Views...),
-            and_v<range<Views const>...>
+    template<typename...Views>
+    CPP_concept_bool cartesian_produce_view_can_const =
+        and_v<range<Views const>...>;
+
+    template<typename IsConst, typename...Views>
+    CPP_concept_fragment(cartesian_produce_view_can_size_, (IsConst, Views...),
+        and_v<common_with<std::uintmax_t, range_size_t<meta::const_if<IsConst, Views>>>...>
     );
-    CPP_def
-    (
-        template(typename IsConst, typename...Views)
-        (concept cartesian_produce_view_can_size)(IsConst, Views...),
-            and_v<sized_range<meta::const_if<IsConst, Views>>...> &&
-                ranges::type<common_type_t<std::uintmax_t, range_size_t<meta::const_if<IsConst, Views>>...>>
+    template<typename IsConst, typename...Views>
+    CPP_concept_bool cartesian_produce_view_can_size =
+        and_v<sized_range<meta::const_if<IsConst, Views>>...> &&
+        CPP_fragment(ranges::cartesian_produce_view_can_size_, IsConst, Views...);
+
+    template<typename IsConst, typename...Views>
+    CPP_concept_fragment(cartesian_produce_view_can_distance_, (IsConst, Views...),
+        and_v<sized_sentinel_for<
+            iterator_t<meta::const_if<IsConst, Views>>,
+            iterator_t<meta::const_if<IsConst, Views>>>...>
     );
-    CPP_def
-    (
-        template(typename IsConst, typename...Views)
-        (concept cartesian_produce_view_can_distance)(IsConst, Views...),
-            cartesian_produce_view_can_size<IsConst, Views...> &&
-            and_v<sized_sentinel_for<
-                iterator_t<meta::const_if<IsConst, Views>>,
-                iterator_t<meta::const_if<IsConst, Views>>>...>
+    template<typename IsConst, typename...Views>
+    CPP_concept_bool cartesian_produce_view_can_distance =
+        cartesian_produce_view_can_size<IsConst, Views...> &&
+        CPP_fragment(ranges::cartesian_produce_view_can_distance_, IsConst, Views...);
+
+    template<typename IsConst, typename...Views>
+    CPP_concept_fragment(cartesian_produce_view_can_random_, (IsConst, Views...),
+        and_v<random_access_iterator<iterator_t<meta::const_if<IsConst, Views>>>...>
     );
-    CPP_def
-    (
-        template(typename IsConst, typename...Views)
-        (concept cartesian_produce_view_can_random)(IsConst, Views...),
-            cartesian_produce_view_can_distance<IsConst, Views...> &&
-            and_v<random_access_iterator<iterator_t<
-                meta::const_if<IsConst, Views>>>...>
-    );
-    CPP_def
-    (
-        template(typename IsConst, typename...Views)
-        (concept cartesian_produce_view_can_bidi)(IsConst, Views...),
-            cartesian_produce_view_can_random<IsConst, Views...> ||
-            and_v<common_range<meta::const_if<IsConst, Views>>...,
-                bidirectional_iterator<iterator_t<
-                    meta::const_if<IsConst, Views>>>...>
+    template<typename IsConst, typename...Views>
+    CPP_concept_bool cartesian_produce_view_can_random =
+        cartesian_produce_view_can_distance<IsConst, Views...> &&
+        CPP_fragment(ranges::cartesian_produce_view_can_random_, IsConst, Views...);
+
+    template<typename IsConst, typename...Views>
+    CPP_concept_fragment(cartesian_produce_view_can_bidi_, (IsConst, Views...),
+        and_v<common_range<meta::const_if<IsConst, Views>>...,
+            bidirectional_iterator<iterator_t<meta::const_if<IsConst, Views>>>...>
     );
+    template<typename IsConst, typename...Views>
+    CPP_concept_bool cartesian_produce_view_can_bidi =
+        cartesian_produce_view_can_random<IsConst, Views...> ||
+        CPP_fragment(ranges::cartesian_produce_view_can_bidi_, IsConst, Views...);
     // clang-format on
 
     template<typename... Views>
@@ -140,7 +148,8 @@ namespace ranges
             friend cursor<true>;
             template<typename T>
             using constify_if = meta::const_if_c<IsConst_, T>;
-            using difference_type = common_type_t<std::intmax_t, range_difference_t<Views>...>;
+            using difference_type =
+                common_type_t<std::intmax_t, range_difference_t<Views>...>;
 
             constify_if<cartesian_product_view> * view_;
             std::tuple<iterator_t<constify_if<Views>>...> its_;
@@ -220,8 +229,8 @@ namespace ranges
                     return;
 
                 auto & i = std::get<N - 1>(its_);
-                auto const my_size =
-                    static_cast<difference_type>(ranges::size(std::get<N - 1>(view_->views_)));
+                auto const my_size = static_cast<difference_type>(
+                    ranges::size(std::get<N - 1>(view_->views_)));
                 auto const first = ranges::begin(std::get<N - 1>(view_->views_));
 
                 auto const idx = static_cast<difference_type>(i - first);
@@ -281,14 +290,15 @@ namespace ranges
                    std::true_type) // common_with
               : cursor(begin_tag{}, view)
             {
-                CPP_assert(common_range<meta::at_c<meta::list<Views...>, 0>>);
+                CPP_assert(
+                    common_range<meta::at_c<meta::list<constify_if<Views>...>, 0>>);
                 std::get<0>(its_) = ranges::end(std::get<0>(view->views_));
             }
             cursor(end_tag, constify_if<cartesian_product_view> * view,
                    std::false_type) // !common_with
               : cursor(begin_tag{}, view)
             {
-                using View0 = meta::at_c<meta::list<Views...>, 0>;
+                using View0 = meta::at_c<meta::list<constify_if<Views>...>, 0>;
                 CPP_assert(!common_range<View0> && random_access_range<View0> &&
                            sized_range<View0>);
                 std::get<0>(its_) += ranges::distance(std::get<0>(view->views_));
@@ -307,15 +317,17 @@ namespace ranges
                 check_at_end_(meta::size_t<sizeof...(Views)>{});
             }
             explicit cursor(end_tag, constify_if<cartesian_product_view> * view)
-              : cursor(end_tag{}, view,
-                       meta::bool_<common_range<meta::at_c<meta::list<Views...>, 0>>>{})
+              : cursor(
+                    end_tag{}, view,
+                    meta::bool_<
+                        common_range<meta::at_c<meta::list<constify_if<Views>...>, 0>>>{})
             {}
             CPP_template(bool Other)( //
                 requires IsConst_ && (!Other)) cursor(cursor<Other> that)
               : view_(that.view_)
               , its_(std::move(that.its_))
             {}
-            common_tuple<range_reference_t<Views>...> read() const
+            common_tuple<range_reference_t<constify_if<Views>>...> read() const
             {
                 return tuple_transform(its_, detail::dereference_fn{});
             }
@@ -432,6 +444,10 @@ namespace ranges
 
         RANGES_INLINE_VARIABLE(cartesian_product_fn, cartesian_product)
     } // namespace views
+
+    /// @}
 } // namespace ranges
 
+#include <range/v3/detail/reenable_warnings.hpp>
+
 #endif
